#!/bin/perl -w
use strict;
use warnings;

sub help{
	if(grep {$_ eq "-h"} @ARGV){
		print "#######################################################################################################################################################\n";
		print "This script can be used to perform function transfer from uniprot-proteome database to protein-coding genes in Cucumber guided by the OPPs results. \n";
		print "Final.anno: 7 columns as: segmentID; 2: ref_geneID; 3: spec_geneID; 4: GO-term; 5: evidence; 6: spec; 7: n-way[2way-n] \n";
		print "Usage: perl function_transfer.pl uniprot_anno_spec.txt ref_spec_n-way[2way-n].good.genemap n-way[2way-n] spec > Final.anno \n";
		print "Input: (1) uniprot_anno.txt: Ten species of uniprot-proteome database were downloaded from http://www.uniprot.org/proteomes/ \n";
		print "       (2) good.genemap: ref_spec_nway_good.genemap obtained by Step3.1 in OPPs_identigy.sh \n";
		print "       (3) n-way: the segments level that supports the inference of OPPs \n";
		print "       (4) species: indicating the functional annotation of origin\n";
		print "Output: Final.anno \n";
		print "#############################################################################################################################################\n";
        }
}

&help();



my %refseqID_geneID; # key: refseqID, value: geneID
my %geneID_goterm; # key: geneID, value: goterm1|goterm2....
my %geneID_goterm_evidence;# key1: geneID; key2: goterm, value: evidence

my ($uniprot_anno,$genemap_good, $nway,$spec)=@ARGV;


###############################################################
# Step1 parsing the uniprot-proteome annotation database file #
###############################################################
&uniprot_anno_parse($uniprot_anno);


###########################################################################
# Step2 parsing the genemap file and output the function_transfer results #
###########################################################################
&genemap_parse($genemap_good,$spec);




sub print_hash{
	my ($ref_hash)=@_;
	foreach (sort keys%$ref_hash){
        	print "$_\t$$ref_hash{$_}\n";
        	}
	}




sub uniprot_anno_parse{
	my ($anno_file)=@_;
	open IN,$anno_file;
	my @all=<IN>;
	close IN;
	my $a=join "", @all;
	my @all_pro=split /\/\/\n/, $a;
	#### collect %refseqID_geneID, %geneID_goterm, %geneID_goterm_evidence information from each item ####
	foreach my $i(@all_pro){
		&item2refseqID($i);
		}
	}



sub genemap_parse{
	my ($genemap, $spec)=@_;
	open IN, $genemap;
	while (<IN>){
		chomp($_);
		&genemap2func($_,$spec);
		}
	close IN;
	}






sub genemap2func{
	(my $line, my $spec)=@_;
	my @all=split /\t/, $line;
	my $id=$refseqID_geneID{$all[2]};
	my @go=split /\|/, $geneID_goterm{$id};
	#### output the function_transder results with 7 columns ####
	#### 1: segmentID; 2: ref_geneID; 3: spec_geneID; 4: GO-term; 5: evidence; 6: spec; 7: n-way[2way-n] ####
	foreach my $i(@go){
		print "$all[0]\t$all[1]\t$all[2]\t$i\t$geneID_goterm_evidence{$id}{$i}\t$spec\t${nway}\n";
		}
	}



sub item2refseqID {
	(my $item)=@_;
	if ($item=~/RefSeq\;/m){
		my @all=split /\n/, $item;
		my $id;
		my $t;
		foreach my $i(@all){
			if ($i=~/^ID/){
				$id=(split /\s+/, $i)[1];
        		}elsif ($i=~/RefSeq\;/){
         			my $refseqID=(split /\;/, $i)[1];
         			$refseqID=~s/\s+//g; $t=$refseqID;
         			$refseqID_geneID{$refseqID}=$id;
        		}elsif ($i=~/GO;/){
		   			my $go=(split /\;/, $i)[1]; $go=~s/\s+//g;
		   			if (!exists $geneID_goterm{$id}){
		   				$geneID_goterm{$id}=$go;
		   			}else{
		      			$geneID_goterm{$id}=$geneID_goterm{$id}."\|".$go;
		   			}
		   			my $evidence=(split /\;/, $i)[3]; $evidence=~s/\s+//g;
		   			$geneID_goterm_evidence{$id}{$go}=$evidence;
		   		}
			}
		}
	}
