# Author by songht.CMB.bnu 
# Date: 2017-12-01
#!/bin/sh
export perl="/userset/perl"


#####################################################################################################################################
# Prepare dataset: genemap_good files produced by OPPs_identification.sh, 10 species of uniprot-proteome database						
# Description: 
#	Step 1 performing function transfer at 2way-n level.			    												
#	Step 2 performing function transfer at n-way level.															
#	Step 3 Combining all results into final.anno.																	
# Dependency tools: function_transfer.pl							 																
# Usage: sh function_transfer.sh 			   																			
#	Input: genemap_good/n-way(2way-n)/good.genemap, uniprot-proteome.txt												
#	Output: Final.anno																									
#	output with 7 Columns as: (1)segmentID;(2)ref_geneID;(3)spec_geneID;(4)GO-terms;(5)Evidence;(6)species;(7)way	
#####################################################################################################################################

export ref="Cucumber" # with Cucumber as reference
export uniprot_proteome="/Userset/uniprot-proteome/"
export genemap_good="/Userset/genemap_good/"
export function_transfer="/Userset/pl_scripts/function_transfer.pl"
export output="/Userset/Output/"
declare spec_array
spec_array=(Cucumerb Cucu-me Water-melo Malus-dome Glyc-max Popu-tr Citrus Brass-ra Arab-th Arab-ly Viti-vi Sola-tu Seta-ti Brach Oryza-brach)

echo 'Step1 perfoming function transfer at 2way-n level'
echo 'Step1 Start at'
date
for n in 5 6 7 9 10 11 12 13 14 15
	do
		export spec="${spec_array[($n-1)]}"
		echo "processing $spec of 2way-$n level"
 		perl $function_transfer ${uniprot_proteome}/uniprot-proteome_$spec.txt ${genemap_good}/2way-n/$ref\_$spec\_2way-${n}_good.genemap 2way-$n $spec | sort -u  > $output/2way-n/$ref\_$spec\_2way-$n.anno 
	done
cat $output/2way-n/*2way-*.anno > $output/2way-n_all.anno
echo 'Step1 Finished at'
date


echo 'Step2 perfoming function transfer at n-way level'
echo 'Step2 Start at'
date
for n in 5 6 7 9 10 11 12 13 14 15
	do
		export spec="${spec_array[($n-1)]}"
		echo "processing $spec of ${n}-way level"
	 	perl $function_transfer ${uniprot_proteome}/uniprot-proteome_$spec.txt ${genemap_good}/n-way/$ref\_$spec\_${n}way_good.genemap ${n}-way $spec | sort -u  > $output/n-way/$ref\_$spec\_${n}-way.anno 
	done
cat $output/n-way/*-way.anno > $output/n-way_all.anno
echo 'Step2 Finished at'
date


echo 'Step3 Combining all results into final.anno'
echo 'Step3 Start at'
date
cat $output/2way-n_all.anno $output/n-way_all.anno > $output/Final.anno

echo 'Step3 Finished at'
date


